from dataclasses import dataclass
from django import forms
from varorm import fields
from varorm.models import Model
from varorm.exceptions import *


@dataclass
class FieldAdminRepresintation:
    field_key: str
    form: forms.Form


class VarORMAdmin:
    def __init__(self, model: Model):
        self.model = model
    
    def _generate_form(self, field: fields.Field, field_key, initial) -> forms.Field:
        context = dict(
            label=field._verbose_name or field_key.capitalize(),
            initial=initial,
            required=field._null,
            **(field.form_field_extra_context or {})
        )
        form_field = None
        if field._choices is not None:
            form_field = forms.ChoiceField(
                choices=field._choices, 
                **context
            )
        elif isinstance(field, fields.IntegerField):
            form_field = forms.IntegerField(
                **context
            )
        elif isinstance(field, fields.DateTimeField):
            form_field = forms.DateTimeField(
                widget=forms.DateTimeInput(attrs={"type": "datetime-local"}),
                **context,
            )
        elif isinstance(field, fields.FloatField):
            form_field = forms.FloatField(
                **context
            )
        elif isinstance(field, fields.BooleanField):
            choices = [
                (True, "Yes"),
                (False, "No")
            ]
            if field._null:
                choices.append(("-", None))
            form_field = forms.ChoiceField(
                choices=choices,
                **context,
            )
        elif isinstance(field, fields.CharField):
            form_field = forms.CharField(
                **context
            )
        elif isinstance(field, fields.TextField):
            form_field = forms.CharField(
                widget=forms.Textarea(),
                **context,
            )
        elif isinstance(field, fields.DateField):
            form_field = forms.DateField(
                widget=forms.DateInput(attrs={"type": "date"}),
                **context,
            )
        
        class Form(forms.Form):
            f = form_field

        return FieldAdminRepresintation(
            field_key=field_key,
            form=Form
        )
        
    def get_forms(self):
        fields = []
        for field_key, field in self.model._fields.items():
            value = None
            try:
                value = getattr(self.model, field_key)
            except VarDoesNotExistException:
                pass
            fields.append(self._generate_form(field, field_key, value))

        return fields


from django.contrib import admin
from django.urls import path
from django.http import HttpRequest
from varorm.dj import views, globals, utils


def create_varorm_model_admin(groups: list):
    class VarormAdmin(admin.ModelAdmin):
        def has_module_permission(self, _):
            return True
        
        def get_model_perms(self, request):
            return {"view": "__all__" in groups or request.user.is_superuser or request.user.groups.filter(name__in=groups).exists()}

    return VarormAdmin


for app, models in globals.MODELS.items():
    django_model = utils.create_fake_model(app, [x[1] for x in models])
    
    groups = list()
    for model in models:
        model_groups = model[1].get_meta().groups or list()
        if model_groups == '__all__':
            model_groups = ['__all__']
        groups.extend(model_groups)
    groups = list(set(groups))

    admin.site.register(django_model, create_varorm_model_admin(groups))



get_urls = admin.site.get_urls

def custom_get_urls():
    urls = get_urls()
    dj_urls = [
        path("dj/<str:app>/", views.get_app_models),
    ]
    urls = dj_urls + urls
    return urls

admin.site.get_urls = custom_get_urls